%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%Compressive Sensing Image Reconstruction%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Coded by Kun Qiu, Jan. 19, 2009

%%%%%%%%%%%%%%%%%%%%%%%%
% clear all
% clc
% close all
% 
% thresh=1e-14;
% RadialNum=44;
%%%%%%%%%%%%%%%%%%%%%%%%

addpath([pwd '/FPC_AS']);
addpath([pwd '/FPC_AS/src']);
addpath([pwd '/FPC_AS/prob_gen']);
addpath([pwd '/FPC_AS/prob_gen/classes']);
addpath([pwd '/subfunctions']);
addpath([pwd '/rwt']);

Img2D=phantom(256);
img_name='Phantom';
[my,mx]=size(Img2D);
m=my*mx;
Imgcntr=Img2D(:);
scale=max(Imgcntr)-min(Imgcntr);

% Sampling operator
% number of radial lines in the Fourier domain
[M,Mh,mh,mhi]=LineMask(RadialNum,mx);
OMEGA=mhi;
smpl_pattern=fftshift(M);

Phi=@(z) A_fhp(z,OMEGA);
Phit=@(z) At_fhp(z,OMEGA,mx);

% taking measurements
y=Phi(Imgcntr);
N=length(y);
Nratio=N/m;
%Effective sensing operator
dwt_L=6;                                     %levels of wavelet transform
wav=daubcqf(2);
H=@(z) H_idwt1d(Phi,z,wav,dwt_L,my,mx);
Ht=@(z) Ht_dwt1d(Phit,z,wav,dwt_L,my,mx);

W=@(z) midwt(z,wav,dwt_L);
Wt=@(z) mdwt(z,wav,dwt_L);

phantom_coeff=Wt(Img2D);
%sparsity level for hard thresholding methods
r_init=length(find(abs(phantom_coeff(:))>0));    

%solve by Back Projection
s_BackProj=Ht(y);
s_BackProj=reshape(s_BackProj,[my mx]);
Img2D_BackProj=W(s_BackProj);
PSNR_BackProj=psnr(Img2D,Img2D_BackProj,scale);

%solve by FPC
tic;
A_FPC=A_operator(H, Ht); 
tau_FPC1=(1e-1)*max(abs(Ht(y)));
M=[];
[s_FPC1,Out_FPC1]=FPC_AS(m,A_FPC,y,tau_FPC1,M);
t_FPC1=toc;
s_FPC1=reshape(s_FPC1,[my mx]);
Img2D_FPC1=W(s_FPC1);
PSNR_FPC1=psnr(Img2D,Img2D_FPC1,scale);

%solve by FPC
tic;
A_FPC=A_operator(H, Ht); 
tau_FPC2=(1e-2)*max(abs(Ht(y)));
M=[];
[s_FPC2,Out_FPC2]=FPC_AS(m,A_FPC,y,tau_FPC2,M);
t_FPC2=toc;
s_FPC2=reshape(s_FPC2,[my mx]);
Img2D_FPC2=W(s_FPC2);
PSNR_FPC2=psnr(Img2D,Img2D_FPC2,scale);


%solve by FPC
tic;
A_FPC=A_operator(H, Ht); 
tau_FPC3=(1e-3)*max(abs(Ht(y)));
M=[];
[s_FPC3,Out_FPC3]=FPC_AS(m,A_FPC,y,tau_FPC3,M);
t_FPC3=toc;
s_FPC3=reshape(s_FPC3,[my mx]);
Img2D_FPC3=W(s_FPC3);
PSNR_FPC3=psnr(Img2D,Img2D_FPC3,scale);


%solve by FPC
tic;
A_FPC=A_operator(H, Ht); 
tau_FPC4=(1e-4)*max(abs(Ht(y)));
M=[];
[s_FPC4,Out_FPC4]=FPC_AS(m,A_FPC,y,tau_FPC4,M);
t_FPC4=toc;
s_FPC4=reshape(s_FPC4,[my mx]);
Img2D_FPC4=W(s_FPC4);
PSNR_FPC4=psnr(Img2D,Img2D_FPC4,scale);

%solve by FPC
tic;
A_FPC=A_operator(H, Ht); 
tau_FPC5=(1e-5)*max(abs(Ht(y)));
M=[];
[s_FPC5,Out_FPC5]=FPC_AS(m,A_FPC,y,tau_FPC5,M);
t_FPC5=toc;
s_FPC5=reshape(s_FPC5,[my mx]);
Img2D_FPC5=W(s_FPC5);
PSNR_FPC5=psnr(Img2D,Img2D_FPC5,scale);

%solve by FPC
tic;
A_FPC=A_operator(H, Ht); 
tau_FPC6=(1e-6)*max(abs(Ht(y)));
M=[];
[s_FPC6,Out_FPC6]=FPC_AS(m,A_FPC,y,tau_FPC6,M);
t_FPC6=toc;
s_FPC6=reshape(s_FPC6,[my mx]);
Img2D_FPC6=W(s_FPC6);
PSNR_FPC6=psnr(Img2D,Img2D_FPC6,scale);

%solve by FPC
tic;
A_FPC=A_operator(H, Ht); 
tau_FPC7=(1e-7)*max(abs(Ht(y)));
M=[];
[s_FPC7,Out_FPC7]=FPC_AS(m,A_FPC,y,tau_FPC7,M);
t_FPC7=toc;
s_FPC7=reshape(s_FPC7,[my mx]);
Img2D_FPC7=W(s_FPC7);
PSNR_FPC7=psnr(Img2D,Img2D_FPC7,scale);


%solve by GPSR_BB
tic;
tau1=(1e-1)*max(abs(Ht(y)));
[s_GPSR1,s_GPSR1db,objective,times,debias_start,mses]=GPSR_BB(y,H,tau1,'AT',Ht,'Debias',1,'ToleranceA',1e-5,'Verbose',0);
t_GPSR1=toc;
s_GPSR1=reshape(s_GPSR1,[my mx]);
Img2D_GPSR1=W(s_GPSR1);
PSNR_GPSR1=psnr(Img2D,Img2D_GPSR1,scale);
if ~isempty(s_GPSR1db)
    s_GPSR1db=reshape(s_GPSR1db,[my mx]);
    Img2D_GPSR1db=W(s_GPSR1db);
    PSNR_GPSR1db=psnr(Img2D,Img2D_GPSR1db,scale);
end

%solve by GPSR_BB
tic;
tau2=(1e-2)*max(abs(Ht(y)));
[s_GPSR2,s_GPSR2db,objective,times,debias_start,mses]=GPSR_BB(y,H,tau2,'AT',Ht,'Debias',1,'ToleranceA',1e-5,'Verbose',0);
t_GPSR2=toc;
s_GPSR2=reshape(s_GPSR2,[my mx]);
Img2D_GPSR2=W(s_GPSR2);
PSNR_GPSR2=psnr(Img2D,Img2D_GPSR2,scale);
if ~isempty(s_GPSR2db)
    s_GPSR2db=reshape(s_GPSR2db,[my mx]);
    Img2D_GPSR2db=W(s_GPSR2db);
    PSNR_GPSR2db=psnr(Img2D,Img2D_GPSR2db,scale);
end


%solve by GPSR_BB
tic;
tau3=(1e-3)*max(abs(Ht(y)));
[s_GPSR3,s_GPSR3db,objective,times,debias_start,mses]=GPSR_BB(y,H,tau3,'AT',Ht,'Debias',1,'ToleranceA',1e-5,'Verbose',0);
t_GPSR3=toc;
s_GPSR3=reshape(s_GPSR3,[my mx]);
Img2D_GPSR3=W(s_GPSR3);
PSNR_GPSR3=psnr(Img2D,Img2D_GPSR3,scale);
if ~isempty(s_GPSR3db)
    s_GPSR3db=reshape(s_GPSR3db,[my mx]);
    Img2D_GPSR3db=W(s_GPSR3db);
    PSNR_GPSR3db=psnr(Img2D,Img2D_GPSR3db,scale);
end

%solve by GPSR_BB
tic;
tau4=(1e-4)*max(abs(Ht(y)));
[s_GPSR4,s_GPSR4db,objective,times,debias_start,mses]=GPSR_BB(y,H,tau4,'AT',Ht,'Debias',1,'ToleranceA',1e-5,'Verbose',0);
t_GPSR4=toc;
s_GPSR4=reshape(s_GPSR4,[my mx]);
Img2D_GPSR4=W(s_GPSR4);
PSNR_GPSR4=psnr(Img2D,Img2D_GPSR4,scale);
if ~isempty(s_GPSR4db)
    s_GPSR4db=reshape(s_GPSR4db,[my mx]);
    Img2D_GPSR4db=W(s_GPSR4db);
    PSNR_GPSR4db=psnr(Img2D,Img2D_GPSR4db,scale);
else
    s_GPSR4db=reshape(s_GPSR4,[my mx]);
    Img2D_GPSR4db=W(s_GPSR4db);
    PSNR_GPSR4db=psnr(Img2D,Img2D_GPSR4db,scale);
end

%solve by GPSR_BB
tic;
tau5=(1e-5)*max(abs(Ht(y)));
[s_GPSR5,s_GPSR5db,objective,times,debias_start,mses]=GPSR_BB(y,H,tau5,'AT',Ht,'Debias',1,'ToleranceA',1e-5,'Verbose',0);
t_GPSR5=toc;
s_GPSR5=reshape(s_GPSR5,[my mx]);
Img2D_GPSR5=W(s_GPSR5);
PSNR_GPSR5=psnr(Img2D,Img2D_GPSR5,scale);
if ~isempty(s_GPSR5db)
    s_GPSR5db=reshape(s_GPSR5db,[my mx]);
    Img2D_GPSR5db=W(s_GPSR5db);
    PSNR_GPSR5db=psnr(Img2D,Img2D_GPSR5db,scale);
end

%solve by GPSR_BB
tic;
tau6=(1e-6)*max(abs(Ht(y)));
[s_GPSR6,s_GPSR6db,objective,times,debias_start,mses]=GPSR_BB(y,H,tau6,'AT',Ht,'Debias',1,'ToleranceA',1e-5,'Verbose',0);
t_GPSR6=toc;
s_GPSR6=reshape(s_GPSR6,[my mx]);
Img2D_GPSR6=W(s_GPSR6);
PSNR_GPSR6=psnr(Img2D,Img2D_GPSR6,scale);
if ~isempty(s_GPSR6db)
    s_GPSR6db=reshape(s_GPSR6db,[my mx]);
    Img2D_GPSR6db=W(s_GPSR6db);
    PSNR_GPSR6db=psnr(Img2D,Img2D_GPSR6db,scale);
end

%solve by GPSR_BB
tic;
tau7=(1e-7)*max(abs(Ht(y)));
[s_GPSR7,s_GPSR7db,objective,times,debias_start,mses]=GPSR_BB(y,H,tau7,'AT',Ht,'Debias',1,'ToleranceA',1e-5,'Verbose',0);
t_GPSR7=toc;
s_GPSR7=reshape(s_GPSR7,[my mx]);
Img2D_GPSR7=W(s_GPSR7);
PSNR_GPSR7=psnr(Img2D,Img2D_GPSR7,scale);
if ~isempty(s_GPSR7db)
    s_GPSR7db=reshape(s_GPSR7db,[my mx]);
    Img2D_GPSR7db=W(s_GPSR7db);
    PSNR_GPSR7db=psnr(Img2D,Img2D_GPSR7db,scale);
end


%solve by ECME
tic;
z_init=zeros(m,1);
[s_ECME,Count_ECME]=ECMEls(H,Ht,y,z_init,r_init,thresh);
t_ECME=toc;
z_ECME=s_ECME+Ht(y-H(s_ECME));
s_ECME=reshape(s_ECME,[my mx]);
Img2D_ECME=W(s_ECME);
PSNR_ECME=psnr(Img2D,Img2D_ECME,scale);
z_ECME=reshape(z_ECME,[my mx]);
Img2D_ECMEz=W(z_ECME);
PSNR_ECMEz=psnr(Img2D,Img2D_ECMEz,scale);

%solve by DORE
tic;
z_init=zeros(m,1);
[s_DORE,A_index_DORE,Count_DORE]=DORE(H,Ht,y,r_init,'Thresh',thresh,'visibility',0);
t_DORE=toc;
z_DORE=s_DORE+Ht(y-H(s_DORE));
s_DORE=reshape(s_DORE,[my mx]);
Img2D_DORE=W(s_DORE);
PSNR_DORE=psnr(Img2D,Img2D_DORE,scale);
z_DORE=reshape(z_DORE,[my mx]);
Img2D_DOREz=W(z_DORE);
PSNR_DOREz=psnr(Img2D,Img2D_DOREz,scale);

%solve by ADOREls
tic;
Len_thresh=500;          
z_init=zeros(m,1);
[s_ADORE,A_index_ADORE,Count_ADORE,Golden_Iter,USS]=ADORE(H,Ht,y,'SearchLen',Len_thresh,'Thresh',thresh);
t_ADORE=toc;
s_ADORE=reshape(s_ADORE,[my mx]);
Img2D_ADORE=W(s_ADORE);
PSNR_ADORE=psnr(Img2D,Img2D_ADORE,scale);

%solve by NIHTls
tic;
[s_NIHT,Count_NIHT]=hard_l0_Mterm_mod(y,H,m,r_init,thresh,'P_trans',Ht);
t_NIHT=toc;
z_NIHT=s_NIHT+Ht(y-H(s_NIHT));
s_NIHT=reshape(s_NIHT,[my mx]);
Img2D_NIHT=W(s_NIHT);
PSNR_NIHT=psnr(Img2D,Img2D_NIHT,scale);
z_NIHT=reshape(z_NIHT,[my mx]);
Img2D_NIHTz=W(z_NIHT);
PSNR_NIHTz=psnr(Img2D,Img2D_NIHTz,scale);

[M,Mh,mh,mhi]=LineMask(RadialNum,mx);
save(['TSP_revision_',img_name,num2str(my),'by',num2str(mx),...
    '_RadialNum',num2str(RadialNum),'_thresh',num2str(thresh),'.mat']);

