% LineMask.m
%
% Returns the indicator of the domain in 2D fourier space for the 
% specified line geometry.
% Usage :  [M,Mh,mi,mhi] = LineMask(L,N)
%
% Written by : Justin Romberg
% Created : 1/26/2004
% Revised : 12/2/2004

function [M,Mh,mi,mhi] = LineMask(L,N,endAngle,iniAngle,thc)

if(nargin<3)
    iniAngle=0;
    endAngle=pi;
elseif(nargin<4);
    iniAngle=0;
end

if(nargin<5)
    thc = linspace(iniAngle, endAngle, L+1);
    thc(end)=[];
end

M = zeros(N);
thc=mod(thc,pi);

% full mask
for ll = 1:L
	if ((thc(ll) <= pi/4) | (thc(ll) > 3*pi/4))
        yr = round(tan(thc(ll))*(-N/2+1:N/2-1))+N/2+1;
        for nn = 1:N-1
            M(yr(nn),nn+1) = 1;
        end
  else 
		xc = round(cot(thc(ll))*(-N/2+1:N/2-1))+N/2+1;
		for nn = 1:N-1
			M(nn+1,xc(nn)) = 1;
		end
	end

end


% upper half plane mask (not including origin)
Mh = zeros(N);
Mh = M;
Mh(N/2+2:N,:) = 0;
Mh(N/2+1,N/2+1:N) = 0;


M = ifftshift(M);
mi = find(M);
Mh = ifftshift(Mh);
mhi = find(Mh);
