% This configuration file is used for RealCT data and the analytically 
% simulated Phantom data. The sinogram is first transformed to frequency 
% domain, if the length of every projection is N, then the reconstructed image 
% should have a size of NxN.

% The y of this configuration file is in frequency domain. Do not have any
% change about the size of frequency domain or size of sinogram or the size of
% reconstructed image.

fprintf('Loading data...\n');
switch lower(imageName)
    case 'phantom'
        Ts=0.1;
        load 'PhantomPb511.mat';
        load 'Phantom512.mat';
%       load 'Mask512Circle.mat'
%       load 'MaskWvlt512.mat'
        load 'PhantomMask512.mat';
        load 'PhantomMaskWvlt512.mat' ;
        %levels of wavelet transform
        if(size(Img2D,1)==512)
            dwt_L=5;    %for  512x512  Phantom image
        else
            dwt_L=6;   %for 1024x1024 Phantom image
        end
        wav=daubcqf(2);
    case 'realct'
        Ts=0.01*0.8;
        
        %Pre-processing data
        center_loc=691; %692;
        dist=8597; %8797; %
        load 'CTdata_220.mat'
        wrap=1;

        wav=daubcqf(6);
        dwt_L=8;        %levels of wavelet transform
%       wav=daubcqf(2);
%       dwt_L=6;        %levels of wavelet transform

        fprintf('Loading Mask %02d...\n',maskNo);
        if(strcmp(subDir,'/CircleMask'))
            load('Mask1024Circle.mat');
            load('MaskWvlt1024CircleL8D6.mat');
        else 
            load(sprintf('RealCTMask_%02d.mat',maskNo));
            load(sprintf('RealCTMaskWvlt_%02d.mat',maskNo));
        end

        CTdata=CTdata_mtrx;
        CTdata=-log(CTdata/max(CTdata(:)));
        CTdata=CTdata(:,end:-1:1);
        CTdata=CTdata(:,[wrap:360 1:wrap-1]);
        CTdata=CTdata(81:2*center_loc-81,:);

%       ds=0.8419;
        paraInc=1;
        [temp, Rpos_Fline,Rang_Fline]=...
            fan2para(CTdata,dist*Ts,'FanSensorGeometry','line',...
            'FanSensorSpacing',Ts,'ParallelCoverage','halfcycle',...
            'Interpolation','pchip','ParallelRotationIncrement',paraInc,...
            'ParallelSensorSpacing',Ts*ds); %fss*ds);  %
        [CTdata, Rpos_Fline,Rang_Fline]=...
            fan2para(CTdata(:,[181:360 1:180]),dist*Ts,...
            'FanSensorGeometry','line',...
            'FanSensorSpacing',Ts,'ParallelCoverage','halfcycle',...
            'Interpolation','pchip','ParallelRotationIncrement',paraInc,...
            'ParallelSensorSpacing',Ts*ds); %fss*ds);  %
        Ts=Ts*ds;
        CTdata=CTdata(end:-1:1,:);
        CTdata=(temp+CTdata)/2;

        % Try to reduce the affect of beam harden
        % filt = (.74 + .26 * cos(pi*(-Num_pixel/2:Num_pixel/2-1)/Num_pixel));
        % CTdata=CTdata.*repmat(filt(:),1,Num_proj);
        % aaa=10^-0.4;
        % CTdata=(exp(aaa*CTdata)-1)/aaa;
end

Num_proj=size(CTdata,2);
theta_full=linspace(0,pi-pi/Num_proj,Num_proj); %The angle of projections
theta=theta_full(theta_idx);
CTdata=CTdata(:,theta_idx);
[Num_pixel,Num_proj]=size(CTdata);

if(any(runList==3))
    temp=2^(ceil(log2(Num_pixel))+1);
else
    temp=2^(ceil(log2(Num_pixel))+0);
end
CTdata=[zeros(ceil((temp-Num_pixel)/2),Num_proj); CTdata;...
    zeros(floor((temp-Num_pixel)/2),Num_proj)];
[Num_pixel,Num_proj]=size(CTdata);

if(any(runList==3))
    mx=Num_pixel/2;
else
    mx=Num_pixel/1;
end
my=mx; m=mx*my;
m_2D=[mx, my];
J=[1,1]*3;                       % NUFFT interpolation neighborhood
K=2.^ceil(log2(m_2D*2));         % oversampling rate

r=pi*linspace(-1,0-2/Num_pixel,Num_pixel/2)';
%The locations of measurements in Frequency plane
xc=r*cos(theta(:)');
yc=r*sin(theta(:)');
temp=[yc(1,:)', xc(1,:)'];
xc=xc(2:end,:); yc=yc(2:end,:);
om=[zeros(Num_proj,2); temp;...
    yc(:), xc(:); -yc(:), -xc(:)];
st=nufft_init(om,m_2D,J,K,m_2D/2,'minmax:kb');
st.Num_pixel=Num_pixel;
st.Num_proj=Num_proj;

CTdata=fftshift(fft(fftshift(CTdata,1)),1)*Ts;
y=real([CTdata(Num_pixel/2+1,:)'; CTdata(1,:)']);
CTdata=CTdata(2:Num_pixel/2,:);
y=[y(:); sqrt(2)*real(CTdata(:)); sqrt(2)*imag(CTdata(:))];
%y=y*sqrt(pi/(2*Ts^3*Num_proj*Num_pixel));
N=length(y);

% Zero freq at f_coeff(Num_pixel/2+1)
f_coeff=designFilter('renliang1',Num_pixel,Ts);
f_coeff=repmat(f_coeff,1,Num_proj);
temp=[f_coeff(Num_pixel/2+1,:)'; f_coeff(1,:)'];
f_coeff=f_coeff(2:Num_pixel/2,:);
f_coeff=[temp(:); f_coeff(:)];

Mask=double(Mask~=0);
%figure; showImg(Mask);
wvltIdx=find(maskk~=0);
p_I=length(wvltIdx);
maskIdx=find(Mask~=0);
p_M=length(maskIdx);

fprintf('Generating Func handles...\n');

%Sampling operator
W=@(z) midwt(z,wav,dwt_L);
Wt=@(z) mdwt(z,wav,dwt_L);

Psi=@(s) PsiFunc(s,W);
PsiM=@(s) PsiFunc(s,W,m,maskIdx,wvltIdx);
Psit=@(s) PsitFunc(s,Wt);
PsiMt=@(s) PsitFunc(s,Wt,m,maskIdx,wvltIdx);

switch lower(PhiPhitMode)
    case 'basic'
        % The most basic methods without anything here
        Phi=@(s) PhiFunc31(s,f_coeff,st,mx,Ts);
        PhiM=@(s) PhiFunc31(s,f_coeff,st,mx,Ts,maskIdx);
        Phit=@(s) PhitFunc31(s,f_coeff,st,mx,Ts);
        PhiMt=@(s) PhitFunc31(s,f_coeff,st,mx,Ts,maskIdx);

        FBP=@(s) FBPFunc3(s,f_coeff,st,Num_pixel,Ts);

    case 'filtered'
        % Sqrt filtered methods
        temp=(N-Num_proj)/2;
        y(Num_proj+1:end-temp)=(y(Num_proj+1:end-temp)+...
            1i*y(Num_proj+temp+1:end))/sqrt(2);
        y(1:end-temp)=y(1:end-temp).*sqrt(f_coeff);
        y(Num_proj+temp+1:end)=sqrt(2)*imag(y(Num_proj+1:end-temp));
        y(Num_proj+1:end-temp)=sqrt(2)*real(y(Num_proj+1:end-temp));

        Phi=@(s) PhiFunc22(s,f_coeff,st,mx,Ts);
        PhiM=@(s) PhiFunc22(s,f_coeff,st,mx,Ts,maskIdx);
        Phit=@(s) PhitFunc22(s,f_coeff,st,Num_pixel,Ts);
        PhiMt=@(s) PhitFunc22(s,f_coeff,st,Num_pixel,Ts,maskIdx);

        FBP=Phit;

    case 'weighted'
        % Fessler's weighted methods
        weight=exp(-y);
        weight=sqrt(weight./max(abs(weight(:))));
        y=y.*weight;

        Phi=@(s) PhiFunc21(s,f_coeff,st,Num_pixel,Ts).*weight(:);
        PhiM=@(s) PhiFunc21(s,f_coeff,st,Num_pixel,Ts,maskIdx).*weight(:);
        Phit=@(s) PhitFunc21(s.*weight(:),f_coeff,st,Num_pixel,Ts);
        PhiMt=@(s) PhitFunc21(s.*weight(:),f_coeff,st,Num_pixel,...
            Ts,maskIdx);
        FBP=@(s) FBPFunc2(s./weight,f_coeff,stAdj,Num_pixel,Ts)...
            /sqrt(pi/Ts/Num_proj);
    otherwise
        fprintf('Wrong mode for PhiPhitMode: %s\n',PhiPhitMode);
        return;
end

H=@(s) Phi(Psi(s));
HM=@(s) PhiM(PsiM(s));
Ht=@(s) Psit(Phit(s));
HMt=@(s) PsiMt(PhiMt(s));

normy=norm(y);

fprintf('Configuration Finished!\n');

if(0)
    testTranspose(Phi,Phit,N,m,'Phi');
    testTranspose(PhiM,PhiMt,N,p_M,'PhiM');
%   testTranspose(Psi,Psit,m,m,'Psi');
%   testTranspose(PsiM,PsiMt,p_M,p_I,'PsiM');
end

c=8.682362e-03;
mc=1;
mc=0.7;
mc=6.8195e-03;    % For full projection
%mc=1.307885e+01;
if(0)
    c=expectHHt(H,Ht,N,m,'H');
    mc=expectHHt(HM,HMt,N,p_I,'HM');
    mc=expectHHt(PhiM,PhiMt,N,p_M,'PhiM');
end

