%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%    Compressive Sensing Reconstruction Example (1-d example)    %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Coded by Kun Qiu (kqiu@iastate.edu), Sept. 21, 2008

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% clear all
% clc
% close all
% SignalType=1;     %Binary signal=0;Gaussian signal=1;
% sigma2=0;      %noise variance
% TrialNum=1;         %number of trials
% T=70;                   %number of spikes
% UseCorrelation=0; %if use correlated sensing matrix, choose 1; otherwise 0
% thresh=1e-14;      %convergence threshold for hard thresholding methods
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Parameter specification
randn('state',120);
% rand('state',18);

addpath(genpath(fullfile(pwd,'FPC_AS')));

N=250;                 %number of measurements
m=500;                %number of underlying signal length

for ll=1:TrialNum

    %Signal and measurement model    
    H=randn(N,m)/sqrt(m);
    
    if UseCorrelation
        alpha=0.9;
        Cov_Mtrx=zeros(N,N);
        AR1_coeff=((1-N):(N-1));
        AR1=alpha.^abs(AR1_coeff);
        for ii=1:N
            Cov_Mtrx(ii,:)=AR1(N+1-ii:2*N-ii);
        end
        L_chol=(chol(Cov_Mtrx))';
        H=L_chol*H;
        
        B_diag=(1+alpha^2)*ones(N,1);
        B_diag(1)=1;
        B_diag(end)=1;
        B_diag=B_diag/(1-alpha^2);
        B_up=-alpha/(1-alpha^2)*ones(N,1);
        B_up(1)=0;
        B_down=-alpha/(1-alpha^2)*ones(N,1);
	    B_down(end)=0;
        B=[B_down,B_diag,B_up];
        Cov_Mtrx_inv=spdiags(B,[-1,0,1],N,N);
%         aa=full(Cov_Mtrx_inv);
%         bb=inv(Cov_Mtrx);
    end
    
    tic;
    HHtInv=inv(H*H');
    Time_Inverse(ll)=toc;
    
    %Sparse signal generation
    s=zeros(m,1);
    q=randperm(m);
    if SignalType==0
        s(q(1:T))=sign(randn(T,1));       %Binary signal
        SignalType_Str='Binary';
    else
        s(q(1:T))=randn(T,1);
        SignalType_Str='Gaussian';
    end
    noi=sqrt(sigma2)*randn(m,1);      %noise

%     q(1:T)
    
    %Measurement
    y=H*(s+noi);
    
    %solve by ECME
    tic;
    z_init=zeros(m,1);
    r_init=T;
    [s_ECME,A_index_ECME,Count_ECME(ll)]=ECME(H,y,r_init,z_init,thresh,HHtInv);
    t_ECME(ll)=toc;
    error_ECME(ll)=norm(s_ECME-s)^2/m;
    Nerror_ECME(ll)=norm(s_ECME-s)/norm(s);
    
%     %solve by ECMEapp
%     tic;
%     z_init=zeros(m,1);
%     r_init=T;
%     [s_ECMEapp,A_index_ECMEapp,Count_ECMEapp]=ECMEapp(H,y,r_init,z_init,thresh);
%     t_ECMEapp(ll)=toc;
%     error_ECMEapp(ll)=norm(s_ECMEapp-s)^2/m;

    %solve by DORE
    tic;
    z_init=zeros(m,1);
    r_init=T;
    [s_DORE,A_index_DORE,Count_DORE(ll)]=DORE(H,[],y,r_init,'Thresh',thresh,'visibility',0,'IsHOrthonormal',0,'HHtInv',HHtInv);
    t_DORE(ll)=toc;
    error_DORE(ll)=norm(s_DORE-s)^2/m;
    Nerror_DORE(ll)=norm(s_DORE-s)/norm(s);
    
    %solve by DOREapp
    tic;
    z_init=zeros(m,1);
    r_init=T;
    [s_DOREapp,A_index_DOREapp,Count_DOREapp(ll)]=DORE(H,[],y,r_init,'Thresh',thresh,'visibility',0,'IsHOrthonormal',1);
    t_DOREapp(ll)=toc;
    error_DOREapp(ll)=norm(s_DOREapp-s)^2/m;
    Nerror_DOREapp(ll)=norm(s_DOREapp-s)/norm(s);
    
    if UseCorrelation
        %solve by DOREapp2
        tic;
        z_init=zeros(m,1);
        r_init=T;
        [s_DOREapp2,A_index_DOREapp2,Count_DOREapp2(ll)]=DORE(H,[],y,r_init,'Thresh',thresh,'visibility',0,'IsHOrthonormal',0,'HHtInv',Cov_Mtrx_inv);
        t_DOREapp2(ll)=toc;
        error_DOREapp2(ll)=norm(s_DOREapp2-s)^2/m;
        Nerror_DOREapp2(ll)=norm(s_DOREapp2-s)/norm(s);
    end

    %solve by ADORE
    tic;
    z_init=zeros(m,1);
    Len_thresh=4;
    [s_ADORE,A_index_ADORE,Count_ADORE(ll)]=ADORE(H,[],y,'Thresh',thresh,'SearchLen',Len_thresh,'IsHOrthonormal',0,'HHtInv',HHtInv);
    t_ADORE(ll)=toc;
    error_ADORE(ll)=norm(s_ADORE-s)^2/m;
    Nerror_ADORE(ll)=norm(s_ADORE-s)/norm(s);
        
    %solve by NIHT
    tic;
    z_init=zeros(m,1);
    r_init=T;
    [s_NIHT,Count_NIHT(ll)]=hard_l0_Mterm_mod(y,H,m,T,thresh,'start_val',z_init);
    t_NIHT(ll)=toc;
    error_NIHT(ll)=norm(s_NIHT-s)^2/m;   
    Nerror_NIHT(ll)=norm(s_NIHT-s)/norm(s);
    
    %solve by FPC1
    tic;
    opts.record=-1;
    tau_FPC1=(1e-1)*max(abs(H'*y));
    M=[];
    [s_FPC1,Out_FPC1]=FPC_AS(m,H,y,tau_FPC1,M,opts);
    t_FPC1(ll)=toc;
    error_FPC1(ll)=norm(s_FPC1-s)^2/m;   
    Nerror_FPC1(ll)=norm(s_FPC1-s)/norm(s);
    
    %solve by FPC2
    tic;
    opts.record=-1;
    tau_FPC2=(1e-2)*max(abs(H'*y));
    M=[];
    [s_FPC2,Out_FPC2]=FPC_AS(m,H,y,tau_FPC2,M,opts);
    t_FPC2(ll)=toc;
    error_FPC2(ll)=norm(s_FPC2-s)^2/m;   
    Nerror_FPC2(ll)=norm(s_FPC2-s)/norm(s);
    
    %solve by FPC3
    tic;
    opts.record=-1;
    tau_FPC3=(1e-3)*max(abs(H'*y));
    M=[];
    [s_FPC3,Out_FPC3]=FPC_AS(m,H,y,tau_FPC3,M,opts);
    t_FPC3(ll)=toc;
    error_FPC3(ll)=norm(s_FPC3-s)^2/m;   
    Nerror_FPC3(ll)=norm(s_FPC3-s)/norm(s);
    
    %solve by FPC4
    tic;
    opts.record=-1;
    tau_FPC4=(1e-4)*max(abs(H'*y));
    M=[];
    [s_FPC4,Out_FPC4]=FPC_AS(m,H,y,tau_FPC4,M,opts);
    t_FPC4(ll)=toc;
    error_FPC4(ll)=norm(s_FPC4-s)^2/m;  
    Nerror_FPC4(ll)=norm(s_FPC4-s)/norm(s); 
    
    %solve by FPC5
    tic;
    opts.record=-1;
    tau_FPC5=(1e-5)*max(abs(H'*y));
    M=[];
    [s_FPC5,Out_FPC5]=FPC_AS(m,H,y,tau_FPC5,M,opts);
    t_FPC5(ll)=toc;
    error_FPC5(ll)=norm(s_FPC5-s)^2/m;   
    Nerror_FPC5(ll)=norm(s_FPC5-s)/norm(s); 
    
    %solve by FPC6
    tic;
    opts.record=-1;
    tau_FPC6=(1e-6)*max(abs(H'*y));
    M=[];
    [s_FPC6,Out_FPC6]=FPC_AS(m,H,y,tau_FPC6,M,opts);
    t_FPC6(ll)=toc;
    error_FPC6(ll)=norm(s_FPC6-s)^2/m;   
    Nerror_FPC6(ll)=norm(s_FPC6-s)/norm(s); 
    
    %solve by FPC7
    tic;
    opts.record=-1;
    tau_FPC7=(1e-7)*max(abs(H'*y));
    M=[];
    [s_FPC7,Out_FPC7]=FPC_AS(m,H,y,tau_FPC7,M,opts);
    t_FPC7(ll)=toc;
    error_FPC7(ll)=norm(s_FPC7-s)^2/m;   
    Nerror_FPC7(ll)=norm(s_FPC7-s)/norm(s); 
    
    %solve by FPC8
    tic;
    opts.record=-1;
    tau_FPC8=(1e-8)*max(abs(H'*y));
    M=[];
    [s_FPC8,Out_FPC8]=FPC_AS(m,H,y,tau_FPC8,M,opts);
    t_FPC8(ll)=toc;
    error_FPC8(ll)=norm(s_FPC8-s)^2/m;   
    Nerror_FPC8(ll)=norm(s_FPC8-s)/norm(s); 
    
    %solve by FPC9
    tic;
    opts.record=-1;
    tau_FPC9=(1e-9)*max(abs(H'*y));
    M=[];
    [s_FPC9,Out_FPC9]=FPC_AS(m,H,y,tau_FPC9,M,opts);
    t_FPC9(ll)=toc;
    error_FPC9(ll)=norm(s_FPC9-s)^2/m;   
    Nerror_FPC9(ll)=norm(s_FPC9-s)/norm(s); 
    
%     %solve by FPC10
%     tic;
%     opts.record=-1;
%     tau_FPC10=(1e-10)*max(abs(H'*y));
%     M=[];
%     [s_FPC10,Out_FPC10]=FPC_AS(m,H,y,tau_FPC10,M,opts);
%     t_FPC10=toc;
%     error_FPC10(ll)=norm(s_FPC10-s)^2/m;   
%     Nerror_FPC10(ll)=norm(s_FPC10-s)/norm(s); 
    
    %solve by GPSR1
    tic;
    tau_GPSR1=(1e-1)*max(abs(H'*y));
    [s_GPSR1,s_GPSR1db]=GPSR_BB(y,H,tau_GPSR1,'Debias',1,'Verbose',0,'ToleranceA',1e-5);
    t_GPSR1(ll)=toc;
    if isempty(s_GPSR1db)
        s_GPSR1db=s_GPSR1;
    end
    error_GPSR1(ll)=norm(s_GPSR1-s)^2/m;
    error_GPSR1db(ll)=norm(s_GPSR1db-s)^2/m;
    Nerror_GPSR1(ll)=norm(s_GPSR1-s)/norm(s); 
    Nerror_GPSR1db(ll)=norm(s_GPSR1db-s)/norm(s); 

    %solve by GPSR2
    tic;
    tau_GPSR2=(1e-2)*max(abs(H'*y));
    [s_GPSR2,s_GPSR2db]=GPSR_BB(y,H,tau_GPSR2,'Debias',1,'Verbose',0,'ToleranceA',1e-5);
    t_GPSR2(ll)=toc;
    if isempty(s_GPSR2db)
        s_GPSR2db=s_GPSR2;
    end
    error_GPSR2(ll)=norm(s_GPSR2-s)^2/m;
    error_GPSR2db(ll)=norm(s_GPSR2db-s)^2/m;
    Nerror_GPSR2(ll)=norm(s_GPSR2-s)/norm(s); 
    Nerror_GPSR2db(ll)=norm(s_GPSR2db-s)/norm(s); 

    %solve by GPSR3
    tic;
    tau_GPSR3=(1e-3)*max(abs(H'*y));
    [s_GPSR3,s_GPSR3db]=GPSR_BB(y,H,tau_GPSR3,'Debias',1,'Verbose',0,'ToleranceA',1e-5);
    t_GPSR3(ll)=toc;
    if isempty(s_GPSR3db)
        s_GPSR3db=s_GPSR3;
    end
    error_GPSR3(ll)=norm(s_GPSR3-s)^2/m;
    error_GPSR3db(ll)=norm(s_GPSR3db-s)^2/m;
    Nerror_GPSR3(ll)=norm(s_GPSR3-s)/norm(s); 
    Nerror_GPSR3db(ll)=norm(s_GPSR3db-s)/norm(s); 

    %solve by GPSR4
    tic;
    tau_GPSR4=(1e-4)*max(abs(H'*y));
    [s_GPSR4,s_GPSR4db]=GPSR_BB(y,H,tau_GPSR4,'Debias',1,'Verbose',0,'ToleranceA',1e-5);
    t_GPSR4(ll)=toc;
    if isempty(s_GPSR4db)
        s_GPSR4db=s_GPSR4;
    end
    error_GPSR4(ll)=norm(s_GPSR4-s)^2/m;
    error_GPSR4db(ll)=norm(s_GPSR4db-s)^2/m;
    Nerror_GPSR4(ll)=norm(s_GPSR4-s)/norm(s); 
    Nerror_GPSR4db(ll)=norm(s_GPSR4db-s)/norm(s); 

    %solve by GPSR5
    tic;
    tau_GPSR5=(1e-5)*max(abs(H'*y));
    [s_GPSR5,s_GPSR5db]=GPSR_BB(y,H,tau_GPSR5,'Debias',1,'Verbose',0,'ToleranceA',1e-5);
    t_GPSR5(ll)=toc;
    if isempty(s_GPSR5db)
        s_GPSR5db=s_GPSR5;
    end
    error_GPSR5(ll)=norm(s_GPSR5-s)^2/m;
    error_GPSR5db(ll)=norm(s_GPSR5db-s)^2/m;
    Nerror_GPSR5(ll)=norm(s_GPSR5-s)/norm(s); 
    Nerror_GPSR5db(ll)=norm(s_GPSR5db-s)/norm(s); 

    %solve by GPSR6
    tic;
    tau_GPSR6=(1e-6)*max(abs(H'*y));
    [s_GPSR6,s_GPSR6db]=GPSR_BB(y,H,tau_GPSR6,'Debias',1,'Verbose',0,'ToleranceA',1e-5);
    t_GPSR6(ll)=toc;
    if isempty(s_GPSR6db)
        s_GPSR6db=s_GPSR6;
    end
    error_GPSR6(ll)=norm(s_GPSR6-s)^2/m;
    error_GPSR6db(ll)=norm(s_GPSR6db-s)^2/m;
    Nerror_GPSR6(ll)=norm(s_GPSR6-s)/norm(s); 
    Nerror_GPSR6db(ll)=norm(s_GPSR6db-s)/norm(s); 

    %solve by GPSR7
    tic;
    tau_GPSR7=(1e-7)*max(abs(H'*y));
    [s_GPSR7,s_GPSR7db]=GPSR_BB(y,H,tau_GPSR7,'Debias',1,'Verbose',0,'ToleranceA',1e-5);
    t_GPSR7(ll)=toc;
    if isempty(s_GPSR7db)
        s_GPSR7db=s_GPSR7;
    end
    error_GPSR7(ll)=norm(s_GPSR7-s)^2/m;
    error_GPSR7db(ll)=norm(s_GPSR7db-s)^2/m;
    Nerror_GPSR7(ll)=norm(s_GPSR7-s)/norm(s); 
    Nerror_GPSR7db(ll)=norm(s_GPSR7db-s)/norm(s); 
    
    %solve by GPSR8
    tic;
    tau_GPSR8=(1e-8)*max(abs(H'*y));
    [s_GPSR8,s_GPSR8db]=GPSR_BB(y,H,tau_GPSR8,'Debias',1,'Verbose',0,'ToleranceA',1e-5);
    t_GPSR8(ll)=toc;
    if isempty(s_GPSR8db)
        s_GPSR8db=s_GPSR8;
    end
    error_GPSR8(ll)=norm(s_GPSR8-s)^2/m;
    error_GPSR8db(ll)=norm(s_GPSR8db-s)^2/m;
    Nerror_GPSR8(ll)=norm(s_GPSR8-s)/norm(s); 
    Nerror_GPSR8db(ll)=norm(s_GPSR8db-s)/norm(s); 

    %solve by GPSR9
    tic;
    tau_GPSR9=(1e-9)*max(abs(H'*y));
    [s_GPSR9,s_GPSR9db]=GPSR_BB(y,H,tau_GPSR9,'Debias',1,'Verbose',0,'ToleranceA',1e-5);
    t_GPSR9(ll)=toc;
    if isempty(s_GPSR9db)
        s_GPSR9db=s_GPSR9;
    end
    error_GPSR9(ll)=norm(s_GPSR9-s)^2/m;
    error_GPSR9db(ll)=norm(s_GPSR9db-s)^2/m;
    Nerror_GPSR9(ll)=norm(s_GPSR9-s)/norm(s); 
    Nerror_GPSR9db(ll)=norm(s_GPSR9db-s)/norm(s); 


%     %solve by clairvoyant estimate
%     x_LS=zeros(m,1);
%     HA=H(:,q(1:T));
%     x_LS(q(1:T))=HA\y;
%     error_LS(ll)=norm(x_LS-s)^2/m;
    
    clc
    display(['Trial ',num2str(ll),' completed (total ',num2str(TrialNum),' trials)']);
end

MSE_ECME=sum(error_ECME)/TrialNum;
NMSE_ECME=sum(Nerror_ECME)/TrialNum;
TimeAvg_ECME=sum(t_ECME)/TrialNum;
CountAvg_ECME=sum(Count_ECME)/TrialNum;

% MSE_ECMEapp=sum(error_ECMEapp)/TrialNum;
% TimeAvg_ECMEapp=sum(t_ECMEapp)/TrialNum;
% CountAvg_ECMEapp=sum(Count_ECMEapp)/TrialNum;

MSE_DORE=sum(error_DORE)/TrialNum;
NMSE_DORE=sum(Nerror_DORE)/TrialNum;
TimeAvg_DORE=sum(t_DORE)/TrialNum;
CountAvg_DORE=sum(Count_DORE)/TrialNum;

MSE_DOREapp=sum(error_DOREapp)/TrialNum;
NMSE_DOREapp=sum(Nerror_DOREapp)/TrialNum;
TimeAvg_DOREapp=sum(t_DOREapp)/TrialNum;
CountAvg_DOREapp=sum(Count_DOREapp)/TrialNum;

if UseCorrelation
    MSE_DOREapp2=sum(error_DOREapp2)/TrialNum;
    NMSE_DOREapp2=sum(Nerror_DOREapp2)/TrialNum;
    TimeAvg_DOREapp2=sum(t_DOREapp2)/TrialNum;
    CountAvg_DOREapp2=sum(Count_DOREapp2)/TrialNum;
end
    
MSE_ADORE=sum(error_ADORE)/TrialNum;
NMSE_ADORE=sum(Nerror_ADORE)/TrialNum;
TimeAvg_ADORE=sum(t_ADORE)/TrialNum;
CountAvg_ADORE=sum(Count_ADORE)/TrialNum;

MSE_NIHT=sum(error_NIHT)/TrialNum;
NMSE_NIHT=sum(Nerror_NIHT)/TrialNum;
TimeAvg_NIHT=sum(t_NIHT)/TrialNum;
CountAvg_NIHT=sum(Count_NIHT)/TrialNum;

MSE_FPC1=sum(error_FPC1)/TrialNum;
NMSE_FPC1=sum(Nerror_FPC1)/TrialNum;
TimeAvg_FPC1=sum(t_FPC1)/TrialNum;

MSE_FPC2=sum(error_FPC2)/TrialNum;
NMSE_FPC2=sum(Nerror_FPC2)/TrialNum;
TimeAvg_FPC2=sum(t_FPC2)/TrialNum;

MSE_FPC3=sum(error_FPC3)/TrialNum;
NMSE_FPC3=sum(Nerror_FPC3)/TrialNum;
TimeAvg_FPC3=sum(t_FPC3)/TrialNum;

MSE_FPC4=sum(error_FPC4)/TrialNum;
NMSE_FPC4=sum(Nerror_FPC4)/TrialNum;
TimeAvg_FPC4=sum(t_FPC4)/TrialNum;

MSE_FPC5=sum(error_FPC5)/TrialNum;
NMSE_FPC5=sum(Nerror_FPC5)/TrialNum;
TimeAvg_FPC5=sum(t_FPC5)/TrialNum;

MSE_FPC6=sum(error_FPC6)/TrialNum;
NMSE_FPC6=sum(Nerror_FPC6)/TrialNum;
TimeAvg_FPC6=sum(t_FPC6)/TrialNum;

MSE_FPC7=sum(error_FPC7)/TrialNum;
NMSE_FPC7=sum(Nerror_FPC7)/TrialNum;
TimeAvg_FPC7=sum(t_FPC7)/TrialNum;

MSE_FPC8=sum(error_FPC8)/TrialNum;
NMSE_FPC8=sum(Nerror_FPC8)/TrialNum;
TimeAvg_FPC8=sum(t_FPC8)/TrialNum;

MSE_FPC9=sum(error_FPC9)/TrialNum;
NMSE_FPC9=sum(Nerror_FPC9)/TrialNum;
TimeAvg_FPC9=sum(t_FPC9)/TrialNum;

% MSE_FPC10=sum(error_FPC10)/TrialNum;
% NMSE_FPC10=sum(Nerror_FPC10)/TrialNum;
% TimeAvg_FPC10=sum(t_FPC10)/TrialNum;

MSE_GPSR1=sum(error_GPSR1)/TrialNum;
MSE_GPSR1db=sum(error_GPSR1db)/TrialNum;
NMSE_GPSR1=sum(Nerror_GPSR1)/TrialNum;
NMSE_GPSR1db=sum(Nerror_GPSR1db)/TrialNum;
TimeAvg_GPSR1=sum(t_GPSR1)/TrialNum;

MSE_GPSR2=sum(error_GPSR2)/TrialNum;
MSE_GPSR2db=sum(error_GPSR2db)/TrialNum;
NMSE_GPSR2=sum(Nerror_GPSR2)/TrialNum;
NMSE_GPSR2db=sum(Nerror_GPSR2db)/TrialNum;
TimeAvg_GPSR2=sum(t_GPSR2)/TrialNum;

MSE_GPSR3=sum(error_GPSR3)/TrialNum;
MSE_GPSR3db=sum(error_GPSR3db)/TrialNum;
NMSE_GPSR3=sum(Nerror_GPSR3)/TrialNum;
NMSE_GPSR3db=sum(Nerror_GPSR3db)/TrialNum;
TimeAvg_GPSR3=sum(t_GPSR3)/TrialNum;

MSE_GPSR4=sum(error_GPSR4)/TrialNum;
MSE_GPSR4db=sum(error_GPSR4db)/TrialNum;
NMSE_GPSR4=sum(Nerror_GPSR4)/TrialNum;
NMSE_GPSR4db=sum(Nerror_GPSR4db)/TrialNum;
TimeAvg_GPSR4=sum(t_GPSR4)/TrialNum;

MSE_GPSR5=sum(error_GPSR5)/TrialNum;
MSE_GPSR5db=sum(error_GPSR5db)/TrialNum;
NMSE_GPSR5=sum(Nerror_GPSR5)/TrialNum;
NMSE_GPSR5db=sum(Nerror_GPSR5db)/TrialNum;
TimeAvg_GPSR5=sum(t_GPSR5)/TrialNum;

MSE_GPSR6=sum(error_GPSR6)/TrialNum;
MSE_GPSR6db=sum(error_GPSR6db)/TrialNum;
NMSE_GPSR6=sum(Nerror_GPSR6)/TrialNum;
NMSE_GPSR6db=sum(Nerror_GPSR6db)/TrialNum;
TimeAvg_GPSR6=sum(t_GPSR6)/TrialNum;

MSE_GPSR7=sum(error_GPSR7)/TrialNum;
MSE_GPSR7db=sum(error_GPSR7db)/TrialNum;
NMSE_GPSR7=sum(Nerror_GPSR7)/TrialNum;
NMSE_GPSR7db=sum(Nerror_GPSR7db)/TrialNum;
TimeAvg_GPSR7=sum(t_GPSR7)/TrialNum;

MSE_GPSR8=sum(error_GPSR8)/TrialNum;
MSE_GPSR8db=sum(error_GPSR8db)/TrialNum;
NMSE_GPSR8=sum(Nerror_GPSR8)/TrialNum;
NMSE_GPSR8db=sum(Nerror_GPSR8db)/TrialNum;
TimeAvg_GPSR8=sum(t_GPSR8)/TrialNum;

MSE_GPSR9=sum(error_GPSR9)/TrialNum;
MSE_GPSR9db=sum(error_GPSR9db)/TrialNum;
NMSE_GPSR9=sum(Nerror_GPSR9)/TrialNum;
NMSE_GPSR9db=sum(Nerror_GPSR9db)/TrialNum;
TimeAvg_GPSR9=sum(t_GPSR9)/TrialNum;

if TrialNum==1
    figure
    set(gcf,'color','white');
    subplot(7,1,1);
    stem(s,'Marker','none','linewidth',2,'color','k');
    axis([1,m,-2,+2]);
    title('True signal');
    box off
    subplot(7,1,2);
    stem(s_ECME,'Marker','none','linewidth',2,'color','k');
    axis([1,m,-2,+2]);
    title('ECME estimated signal');
    box off
    subplot(7,1,3);
    stem(s_DORE,'Marker','none','linewidth',2,'color','k');
    axis([1,m,-2,+2]);
    title('DORE estimated signal');
    box off
    subplot(7,1,4);
    stem(s_ADORE,'Marker','none','linewidth',2,'color','k');
    axis([1,m,-2,+2]);
    title('ADORE estimated signal');
    box off
    subplot(7,1,5);
    stem(s_NIHT,'Marker','none','linewidth',2,'color','k');
    axis([1,m,-2,+2]);
    title('NIHT estimated signal');
    box off
    subplot(7,1,6);
    stem(s_FPC3,'Marker','none','linewidth',2,'color','k');
    axis([1,m,-2,+2]);
    title('FPC3 estimated signal');
    box off
    subplot(7,1,7);
    stem(s_GPSR3db,'Marker','none','linewidth',2,'color','k');
    axis([1,m,-2,+2]);
    title('GPSR3db estimated signal');
    box off
end

clear H HA Phi Psi

save([SignalType_Str,'_Corr',num2str(UseCorrelation),'_TrialNum',...
    num2str(TrialNum),'_N',num2str(N),'_T',num2str(T),'_sigma2-',...
    num2str(sigma2),'.mat']);

